@@ -39,12 +39,13 @@ Utilities for dealing with Signed JSON Web Tokens.
 
 """
 
-import json
+import os
 import struct
 import hashlib
 import M2Crypto
 
-from vep.utils import decode_bytes
+from vep.utils import decode_bytes, encode_bytes
+from vep.utils import decode_json_bytes, encode_json_bytes
 
 
 class JWT(object):
@@ -54,22 +55,34 @@ class JWT(object):
     constructor is only for internal purposes.
     """
 
-    @classmethod
-    def parse(cls, jwt):
-        """Parse a JWT from a string."""
-        algorithm, payload, signature = jwt.split(".")
-        signed_data = algorithm + "." + payload
-        algorithm = json.loads(decode_bytes(algorithm))["alg"]
-        payload = json.loads(decode_bytes(payload))
-        signature = decode_bytes(signature)
-        return cls(algorithm, payload, signature, signed_data)
-
     def __init__(self, algorithm, payload, signature, signed_data):
         self.algorithm = algorithm
         self.payload = payload
         self.signature = signature
         self.signed_data = signed_data
 
+    @classmethod
+    def parse(cls, jwt):
+        """Parse a JWT from a string."""
+        algorithm, payload, signature = jwt.split(".")
+        signed_data = algorithm + "." + payload
+        try:
+            algorithm = decode_json_bytes(algorithm)["alg"]
+        except KeyError:
+            raise ValueError("badly formed JWT")
+        payload = decode_json_bytes(payload)
+        signature = decode_bytes(signature)
+        return cls(algorithm, payload, signature, signed_data)
+
+    @classmethod
+    def generate(cls, payload, key):
+        """Generate and sign a JWT for a dict payload."""
+        alg = key.__class__.__name__[:-3]
+        algorithm = encode_json_bytes({"alg": alg})
+        payload = encode_json_bytes(payload)
+        signature = encode_bytes(key.sign(".".join((algorithm, payload))))
+        return ".".join((algorithm, payload, signature))
+
     def check_signature(self, key_data):
         """Check that the JWT was signed with the given key."""
         if not self.algorithm.startswith(key_data["algorithm"]):
@@ -80,11 +93,14 @@ class JWT(object):
 
 def load_key(algorithm, key_data):
     """Load a Key object from the given data."""
-    assert algorithm.isalnum()
+    if not algorithm.isalnum():
+        msg = "unknown signing algorithm: %s" % (algorithm,)
+        raise ValueError(msg)
     try:
         key_class = globals()[algorithm + "Key"]
     except KeyError:
-        raise ValueError("unknown signing algorithm: %s" % (algorithm,))
+        msg = "unknown signing algorithm: %s" % (algorithm,)
+        raise ValueError(msg)
     return key_class(key_data)
 
 
@@ -94,6 +110,9 @@ class Key(object):
     def verify(self, signed_data, signature):
         raise NotImplementedError
 
+    def sign(self, data):
+        raise NotImplementedError
+
 
 #
 #  RSA keys, implemented using the RSA support in M2Crypto.
@@ -113,9 +132,7 @@ class RSKey(object):
         digest = self.HASHMOD(signed_data).digest()
         try:
             return self.rsa.verify(digest, signature, self.HASHNAME)
-        except M2Crypto.RSA.RSAError, e:
-            if "bad signature" not in str(e):
-                raise
+        except M2Crypto.RSA.RSAError:
             return False
 
 
@@ -139,7 +156,7 @@ class RS256Key(RSKey):
 
 #
 #  DSA keys, implemented by hand because I haven't figured out how to
-#  map what M2Crypto provides onto that the browserid.org server does.
+#  map what M2Crypto provides onto what the browserid.org server does.
 #
 
 class DSKey(object):
@@ -148,10 +165,14 @@ class DSKey(object):
     HASHMOD = None
 
     def __init__(self, data):
-        self.p = int(data["p"], 16)
-        self.q = int(data["q"], 16)
-        self.g = int(data["g"], 16)
-        self.y = int(data["y"], 16)
+        self.p = long(data["p"], 16)
+        self.q = long(data["q"], 16)
+        self.g = long(data["g"], 16)
+        self.y = long(data["y"], 16)
+        if "x" in data:
+            self.x = long(data["x"], 16)
+        else:
+            self.x = None
 
     def verify(self, signed_data, signature):
         p, q, g, y = self.p, self.q, self.g, self.y
@@ -160,21 +181,45 @@ class DSKey(object):
         signature = signature.rjust(hexlength * 2, "0")
         if len(signature) != hexlength * 2:
             return False
-        r = int(signature[:hexlength], 16)
-        s = int(signature[hexlength:], 16)
+        r = long(signature[:hexlength], 16)
+        s = long(signature[hexlength:], 16)
         if r <= 0 or r >= q:
             return False
         if s <= 0 or s >= q:
             return False
         w = modinv(s, q)
-        u1 = (int(self.HASHMOD(signed_data).hexdigest(), 16) * w) % q
+        u1 = (long(self.HASHMOD(signed_data).hexdigest(), 16) * w) % q
         u2 = (r * w) % q
         v = ((pow(g, u1, p) * pow(y, u2, p)) % p) % q
         return (v == r)
 
+    def sign(self, data):
+        p, q, g, y, x = self.p, self.q, self.g, self.y, self.x
+        if not x:
+            raise ValueError("private key not present")
+        # We need to do lots of if-not-this-then-start-over type tests.
+        while True:
+            k = long(os.urandom(self.BITLENGTH / 8).encode("hex"), 16) % q
+            if k == 0:
+                continue
+            r = pow(g, k, p) % q
+            if r == 0:
+                continue
+            h = (long(self.HASHMOD(data).hexdigest(), 16) + (x * r)) % q
+            s = (modinv(k, q) * h) % q
+            if s == 0:
+                continue
+            break
+        assert 0 < r < q
+        assert 0 < s < q
+        bytelength = self.BITLENGTH / 8
+        r_bytes = int2mpint(r)[4:].rjust(bytelength, "\x00")
+        s_bytes = int2mpint(s)[4:].rjust(bytelength, "\x00")
+        return r_bytes + s_bytes
+
 
 class DS128Key(DSKey):
-    BITLENGTH = 128
+    BITLENGTH = 160
     HASHMOD = hashlib.sha1
 
 
@@ -189,7 +234,7 @@ class DS256Key(DSKey):
 
 
 def int2mpint(x, pad=None):
-    """Convert integer into OpenSSL's MPINT format."""
+    """Convert a Python integer into a string in OpenSSL's MPINT format."""
     # The horror...the horror...
     bytes = []
     while x:
@@ -203,7 +248,7 @@ def int2mpint(x, pad=None):
 
 
 def modinv(a, m):
-    """Find the inverse of a modulo m."""
+    """Find the modular inverse of a, with modulus m."""
     # This is a transliteration of the algorithm as it was described
     # to me by Wikipedia, using the Extended Euclidean Algorithm.
     x = 0
